@@ -30,7 +30,7 @@ class BeltController(BeltCommunicationDelegate):
 
         # Delegates
         self._delegate = delegate
-        self._system_handlers = []
+        self._notifications_handlers = []
 
         # Connection state
         self._connection_state = BeltConnectionState.DISCONNECTED
@@ -180,6 +180,41 @@ class BeltController(BeltCommunicationDelegate):
         """
         return self._default_intensity
 
+    def set_default_intensity(self, intensity, vibration_feedback=False, wait_ack=False) -> bool:
+        """
+        Sets the default intensity for belt signals such as compass.
+
+        Note: The minimum default intensity is 5%. Any request to change the intensity below 5% will result in setting
+        this minimum intensity.
+
+        :param int intensity: The default intensity to set in range [0-100].
+        :param bool vibration_feedback: True to start a vibration informing about the new intensity, False for no
+        vibration feedback.
+        :param bool wait_ack: True to wait for mode change acknowledgment.
+        :return: 'True' if the request has been sent successfully.
+        :raise TimeoutError: If the timeout period is reached when waiting for acknowledgment.
+        """
+        if self._connection_state != BeltConnectionState.CONNECTED:
+            self.logger.warning("BeltController: Cannot set the default intensity when not connected.")
+            return False
+        if intensity < 0:
+            intensity = 0
+        if intensity > 100:
+            intensity = 100
+        if wait_ack:
+            write_result = self.write_gatt(
+                navibelt_param_request_char,
+                bytes([0x01, 0x82, intensity, 0x00, (0x01 if vibration_feedback else 0x00)]),
+                navibelt_param_notification_char,
+                b'\x01\x02')
+        else:
+            write_result = self.write_gatt(
+                navibelt_param_request_char,
+                bytes([0x01, 0x82, intensity, 0x00, (0x01 if vibration_feedback else 0x00)]))
+        if write_result == 2:
+            raise TimeoutError("Timeout period reached when changing the belt mode.")
+        return write_result == 0
+
     def write_gatt(self, gatt_char, data, ack_char=None, ack_data=None, timeout_sec=WAIT_ACK_TIMEOUT_SEC) -> int:
         """
         Sends data to a GATT characteristic.
@@ -285,19 +320,19 @@ class BeltController(BeltCommunicationDelegate):
         self._ack_event.clear()
         return 0
 
-    def add_system_handler(self, handler):
+    def add_notifications_handler(self, handler):
         """
-        Adds a system handler.
-        :param BeltSystemHandler handler: The system handler to add.
+        Adds a notifications handler.
+        :param BeltNotificationsHandler handler: The notifications handler to add.
         """
-        self._system_handlers.append(handler)
+        self._notifications_handlers.append(handler)
 
-    def remove_system_handler(self, handler):
+    def remove_notifications_handler(self, handler):
         """
-        Removes a system handler.
-        :param BeltSystemHandler handler: The handler to remove
+        Removes a notifications handler.
+        :param BeltNotificationsHandler handler: The handler to remove
         """
-        self._system_handlers.remove(handler)
+        self._notifications_handlers.remove(handler)
 
     def set_orientation_notifications(self, enabled) -> bool:
         """
@@ -339,6 +374,10 @@ class BeltController(BeltCommunicationDelegate):
         :param str suffix: The suffix of the belt to set.
         :return: 'True' if the request has been sent successfully.
         """
+        # TODO To be moved in diagnostic app
+        # TODO Adds parameter wait_ack
+        # TODO Raise ValueError If a parameter value is illegal.
+        # TODO Reset name if suffix is None
         if self._connection_state != BeltConnectionState.CONNECTED:
             return False
         encoded_suffix = None
@@ -360,6 +399,47 @@ class BeltController(BeltCommunicationDelegate):
         if isinstance(self._communication_interface, BleInterface):
             self._communication_interface.close()
 
+    def set_pairing_requirement(self, pairing_required, save=True, wait_ack=False) -> bool:
+        """
+        Sets the pairing requirement of the belt. The pairing requirement determines if the belt requires pairing or not
+        for connections. When pairing is disabled, any device can connect to the belt without pairing. When pairing is
+        enable, the belt only accept connection from paired devices. To pair a device when pairing is enabled, the
+        pairing mode must be started on the belt by pressing the home button for at least 3 seconds, then the device to
+        connect must request pairing.
+
+        It is recommended to enable pairing for a general usage of the belt and to disable it only for testing purpose.
+
+        :param bool pairing_required: 'True' to require pairing.
+        :param bool save: 'True' to save the setting on the belt, False for a temporary setting.
+        :param bool wait_ack: 'True' to wait for acknowledgment.
+        :return: 'True' if the request has been sent successfully.
+        """
+        if self._connection_state != BeltConnectionState.CONNECTED:
+            self.logger.warning("BeltController: Cannot set pairing requirement when not connected.")
+            return False
+        if wait_ack:
+            write_result = self.write_gatt(
+                navibelt_param_request_char,
+                bytes([
+                    0x11,
+                    0x25,
+                    (0x01 if save else 0x00),
+                    (0x01 if pairing_required else 0x00)]),
+                navibelt_param_notification_char,
+                b'\x10\x25')
+        else:
+            write_result = self.write_gatt(
+                navibelt_param_request_char,
+                bytes([
+                    0x11,
+                    0x25,
+                    (0x01 if save else 0x00),
+                    (0x01 if pairing_required else 0x00)]),
+                navibelt_param_notification_char)
+        if write_result == 2:
+            raise TimeoutError("Timeout period reached when setting pairing requirement.")
+        return write_result == 0
+
     def vibrate_at_magnetic_bearing(
             self,
             bearing,
@@ -883,6 +963,19 @@ class BeltController(BeltCommunicationDelegate):
         except:
             pass
 
+    def _notify_pairing_requirement(self, pairing_required):
+        """
+        Notifies the delegate of the pairing requirement state.
+        :param pairing_required: 'True' if pairing is required.
+        """
+        if (self._connection_state == BeltConnectionState.DISCONNECTED or
+                self._connection_state == BeltConnectionState.DISCONNECTING):
+            return
+        try:
+            self._delegate.on_pairing_requirement_notified(pairing_required)
+        except:
+            pass
+
     def _notify_belt_orientation(self, packet):
         """Notifies the belt orientation to the delegate.
 
@@ -1105,6 +1198,9 @@ class BeltController(BeltCommunicationDelegate):
                 elif data[1] == 0x03:
                     # Compass accuracy signal state
                     self._notify_compass_accuracy_signal_state(data[2])
+                elif data[1] == 0x25:
+                    # Pairing requirement
+                    self._notify_pairing_requirement(data[2] != 0)
 
         # Belt orientation
         if gatt_char == navibelt_orientation_data_char:
@@ -1152,7 +1248,7 @@ class BeltController(BeltCommunicationDelegate):
                 self._ack_event.set()
 
         # Inform system handlers
-        for handler in self._system_handlers:
+        for handler in self._notifications_handlers:
             try:
                 handler.on_gatt_char_notified(gatt_char, data)
             except:
@@ -1294,6 +1390,13 @@ class BeltControllerDelegate:
         """
         pass
 
+    def on_pairing_requirement_notified(self, pairing_required):
+        """ Called when the pairing requirement has been changed or notified.
+
+        :param pairing_required: 'True' if the pairing is required.
+        """
+        pass
+
     def on_belt_orientation_notified(self, heading, is_orientation_accurate, extra):
         """ Called when the orientation of the belt has been notified.
 
@@ -1326,7 +1429,7 @@ class BeltConnectionError(Exception):
         super().__init__(message)
 
 
-class BeltSystemHandler:
+class BeltNotificationsHandler:
     """
     Handler interface for belt debug and test.
     """
@@ -1335,7 +1438,7 @@ class BeltSystemHandler:
         """
         Called when a GATT notification has been received or a characteristic has been read.
 
-        :param GattCharacteristic gatt_char:
+        :param GattCharacteristic gatt_char: The GATT characteristic.
         :param bytes data: The data received.
         """
         pass
